@@ -139,6 +139,9 @@ sub store_device {
       $device->set_column( vtp_domain => (values %$vtpdomains)[-1] );
   }
 
+  my $hostname = hostname_from_ip($device->ip);
+  $device->set_column( dns => $hostname ) if $hostname;
+
   my @properties = qw/
     snmp_ver
     description uptime contact name location
@@ -428,7 +431,8 @@ sub store_vlans {
   my $interfaces        = $snmp->interfaces;
 
   # build device port vlans suitable for DBIC
-  my @portvlans;
+  my @portvlans = ();
+  my %port_vseen = ();
   foreach my $entry (keys %$i_vlan_membership) {
       my $port = $interfaces->{$entry};
       next unless defined $port;
@@ -436,6 +440,8 @@ sub store_vlans {
       my $type = $i_vlan_type->{$entry};
 
       foreach my $vlan (@{ $i_vlan_membership->{$entry} }) {
+          next if ++$port_vseen{$vlan} > 1;
+
           my $native = ((defined $i_vlan->{$entry}) and ($vlan eq $i_vlan->{$entry})) ? "t" : "f";
           push @portvlans, {
               port => $port,
